//
//  ThoMoOFFUtilities.h
//  ThoMoOFFViewer
//
//  Created by Thorsten Karrer on 17.8.11.
//  Copyright 2011 Media Computing Group, RWTH Aachen University. All rights reserved.
//

#import <Foundation/Foundation.h>

#pragma mark -
#pragma mark Geometric Primitives

typedef struct {
	GLfloat x;
	GLfloat y;
	GLfloat z;
} ThoMoOFFPoint;

extern const ThoMoOFFPoint ThoMoOFFPointZero;
extern ThoMoOFFPoint ThoMoOFFPointCrossProduct(ThoMoOFFPoint A, ThoMoOFFPoint B);
extern ThoMoOFFPoint ThoMoOFFPointNormalize(ThoMoOFFPoint A);
extern GLfloat ThoMoOffPointEuklidNorm(ThoMoOFFPoint A);
extern ThoMoOFFPoint ThoMoOFFPointScale(ThoMoOFFPoint A, GLfloat s);
extern GLfloat ThoMoOffPointScalarProduct(ThoMoOFFPoint A, ThoMoOFFPoint B);
extern ThoMoOFFPoint ThoMoOFFPointSubtract(ThoMoOFFPoint A, ThoMoOFFPoint B);
extern ThoMoOFFPoint ThoMoOFFPointAdd(ThoMoOFFPoint A, ThoMoOFFPoint B);

typedef struct {
	GLfloat r;
	GLfloat g;
	GLfloat b;
	GLfloat a;
} ThoMoOFFColor;

extern const ThoMoOFFColor ThoMoOFFColorInvalid;
extern const ThoMoOFFColor ThoMoOFFColorGray;
extern BOOL ThoMoOFFColorIsValid(ThoMoOFFColor C);

typedef struct {
	GLfloat u;
	GLfloat v;
} ThoMoOFFTexCoords;

typedef struct {
	GLfloat nx;
	GLfloat ny;
	GLfloat nz;
} ThoMoOFFNormal;

extern ThoMoOFFNormal ThoMoOFFNormalAdd(ThoMoOFFNormal A, ThoMoOFFNormal B);
extern ThoMoOFFNormal ThoMoOFFNormalScale(ThoMoOFFNormal A, GLfloat s);
extern GLfloat ThoMoOffNormalScalarProduct(ThoMoOFFNormal A, ThoMoOFFNormal B);
extern GLfloat ThoMoOffNormalEuklidNorm(ThoMoOFFNormal A);
extern ThoMoOFFNormal ThoMoOFFNormalNormalize(ThoMoOFFNormal A);

@interface ThoMoOFFVertex : NSObject
@property ThoMoOFFPoint		position;
@property ThoMoOFFNormal	normal;
@property ThoMoOFFColor		color;
@property ThoMoOFFTexCoords texCoords;
@property (readonly) NSString *description;
@end

@interface ThoMoOFFFace : NSObject
@property ThoMoOFFColor color;
@property (strong) NSArray *vertexIndices;
@property (readonly) NSString *description;
@end


#pragma mark -
#pragma mark Constants

enum ThoMoOFFVertexAttributes 
{
	kThoMoOFFVertexAttributeInvalid = -1,
	kThoMoOFFVertexAttributePosition,
	kThoMoOFFVertexAttributeNormal,
	kThoMoOFFVertexAttributeColor,
	kThoMoOFFVertexAttributeTexCoord,
	kThoMoOFFVertexAttributeCount
};

extern const GLint sizeofInterleavedVertexAttributePack;
extern const GLint kThoMoOFFNumberOfComponentsInPoint;
extern const GLint kThoMoOFFNumberOfComponentsInNormal;
extern const GLint kThoMoOFFNumberOfComponentsInColor;
extern const GLint kThoMoOFFNumberOfComponentsInTexCoord;

#pragma mark -
#pragma mark Helper Functions

@interface ThoMoOFFUtilities : NSObject

// VBOs
+ (void) createInterleavedVertexAttributeBuffer:(GLfloat **)bufferHandle fromOFFVertices:(NSArray *)theArray;
+ (void) createVertexAttributeBuffersPosition:(GLfloat **)positionBufferHandle normal:(GLfloat **)normalBufferHandle color:(GLfloat **)colorBufferHandle texCoord:(GLfloat **)texCoordBufferHandle fromOffVertices:(NSArray *)theArray;

// IBOs
+ (void) createIndexBuffer:(GLuint **)bufferHandle numFaces:(GLuint *)numFaces fromOffFaces:(NSArray *)theArray;

@end
